/*
  Name: 3DMath.h
  Author: Carsten Haubold (CarstenHaubold@web.de)
  Date: 14.02.05 20:39
  Description: Lots of methods to calculate something or to handle vectors/vertices
*/

#include <math.h>
#include "texture.h"


#ifndef __3dmath
#define __3dmath

#define PI 3.141592654
#define DEG2RAD PI/180
#define RAD2DEG 180/PI

//Vector- bzw. Vertexstruktur
typedef struct{
        float x,y,z; //3 components of a vector/vertex
} Vertex;

typedef struct{
        float s,t; //2 Components of Texture coordinates
} TexCoord;

typedef struct{
        float x,y,z,s1,t1,s2,t2; //Vertex+2 texCoords
} VertexT2;

typedef struct{
        float x,y,z,s1,t1; //Vertex+texCoord
} VertexT1;


//*************************************************
//Funktionen
//***************************************************


Vertex newV(float x,float y, float z){
       Vertex t;
       t.x=x;
       t.y=y;
       t.z=z;
       return t;
}

Vertex add(Vertex v1,Vertex v2){
	Vertex temp;
	temp.x=v1.x+v2.x;
	temp.y=v1.y+v2.y;
	temp.z=v1.z+v2.z;
	return temp;
}

Vertex sub(Vertex v1,Vertex v2){
	Vertex temp;
	temp.x=v1.x-v2.x;
	temp.y=v1.y-v2.y;
	temp.z=v1.z-v2.z;
	return temp;
}

Vertex multScalar(Vertex v1,float s){
	Vertex temp;
	temp.x=v1.x*s;
	temp.y=v1.y*s;
	temp.z=v1.z*s;

	return temp;
}

Vertex div(Vertex v1,Vertex v2){
       Vertex temp;
       temp.x=v1.x/v2.x;
       temp.y=v1.y/v2.y;
       temp.z=v1.z/v2.z;
       return temp;
}



float vlength(Vertex v){
	return sqrt(v.x*v.x+v.y*v.y+v.z*v.z);
}

float vlength(Vertex *v){
	return sqrt(v->x*v->x+v->y*v->y+v->z*v->z);
}


void normalizeVec(Vertex *v){
	float l;
    l=vlength(v);
	v->x/=l;
	v->y/=l;
	v->z/=l;
}

Vertex normalizeVec(Vertex v){
    float l;
	Vertex vt;
	l=vlength(v);
	vt.x=v.x/l;
	vt.y=v.y/l;
	vt.z=v.z/l;

    return vt;
}

float dotProduct(Vertex v1, Vertex v2){

      Vertex v1t=normalizeVec(v1);
      Vertex v2t=normalizeVec(v2);

      return RAD2DEG*acos((v1t.x*v2t.x)+(v1t.y*v2t.y)+(v1t.z*v2t.z));
}

Vertex calcNormal(Vertex p1,Vertex p2,Vertex p3){
	Vertex normal;
	Vertex vec1,vec2;
	vec1=sub(p1,p2);
	vec2=sub(p1,p3);

    normal.x=vec1.y*vec2.z-vec1.z*vec2.y;
	normal.y=vec1.z*vec2.x-vec1.x*vec2.z;
	normal.z=vec1.x*vec2.y-vec1.y*vec2.x;

	normalizeVec(&normal);

	return normal;
}

// CalculateNormal()
// desc: given 3 points, calculates the normal to the plane described by the points
void CalculateNormal( Vertex p1, Vertex p2, Vertex p3 )
{
   Vertex result=calcNormal(p1,p2,p3);

   // normalize and specify the normal
   glNormal3f(result.x, result.y, result.z);
}


Vertex calcCenter(Vertex p1,Vertex p2, Vertex p3){
       Vertex center=(add(p1,add(p2,p3)));
       center.x/=3;
       center.y/=3;
       center.z/=3;
       return center;
}


//**********************************************************
//Procedures to use the Vertex/vecors with OpenGL
//**********************************************************


void glRotatev2(Vertex v){
     float x,y,z;
     x=acos(dotProduct(newV(1,0,0),v));
     y=acos(dotProduct(newV(0,1,0),v));
     z=acos(dotProduct(newV(0,0,1),v));
     glRotatef(x,1,0,0);
     glRotatef(y,0,1,0);
     glRotatef(z,0,0,1);
}

void glRotatev(Vertex v){
     glRotatef(v.x,1,0,0);
     glRotatef(v.y,0,1,0);
     glRotatef(v.z,0,0,1);
}

void glRotatef2(float x, float y, float z){
     glRotatef(x,1,0,0);
     glRotatef(y,0,1,0);
     glRotatef(z,0,0,1);
}

void glTranslatev(Vertex v){
     glTranslatef(v.x,v.y,-v.z);
}


void glVertexV(Vertex v){
     glVertex3f(v.x,v.y,v.z);
}




#endif
