#ifndef _Terrain
#define _Terrain

#include <gl\gl.h>
#include "texture.h"
#include "3dmath.h"
#include "logging.h"
#include "md2.h"
#include "skybox.h"
#include "water.h"


#define SECNR 8
#define VERTNR 8
#define TOLERANCE 0.03




class Terrain{
      public:
      texture_t detail;
      texture_t floor;
      int hscale, max, scale, step, MaxView,tertexNR;
      short *points;  //storing the height data for every point
      short *pointssmall; //storing the data for far distances
      int maxsmall, scalesmall, dif, fogging;
      CMD2Model *models;
      int modelnr;
      texture_t shadows;
            


      // Funktionen um mit Landschaftsdaten umzugehen
      int aviable(int x){
          if(x>=max) return max-1;
          if(x<0) return 0;
          return x;
      }

      int aviable_s(int x){
          if(x>=maxsmall) return maxsmall-1;
          if(x<0) return 0;
          return x;
      }

      short getHeight(int x, int z){
          return points[aviable(z)*max+aviable(x)];
      }

      short getHeightSmall(int x, int z){
          return pointssmall[aviable_s(z)*maxsmall+aviable_s(x)];
      }
      
      //**********************************************************
      // Calculates an interpolated Height point of a specified point
      //**********************************************************

      float getIHeight(float x, float z){
            float temp[4];
            float xi=int(x);
            float zi=int(z);
            temp[0]=(getHeight(xi,zi));
            temp[1]=(getHeight(xi+1,zi));
            temp[2]=(getHeight(xi+1,zi+1));
            temp[3]=(getHeight(xi,zi+1));
            
            float tx=x-xi;
            float tz=z-zi;
            float txtz=tx*tz;            
            //bilinear interpolation to get the current height above the surface
            return temp[0]*(1-tz-tx+txtz)+temp[1]*(tx-txtz)+temp[2]*txtz+temp[3]*(tz-txtz);
      }
      
      //**********************************************************
      // calls getIHeight after scaling down the position 
      //**********************************************************

      
      float getIUSHeight(float x, float z){
            return getIHeight(x/scale,z/scale);
      }



      void calcCarPos(Vertex pos, Vertex dir, float zaxis, float xaxis, float *temp){
            temp[5]=getIHeight(pos.x/scale,pos.z/scale);      
            
            
            temp[0]=getIUSHeight(pos.x+(dir.x),pos.z+(dir.z));
            temp[1]=getIUSHeight(pos.x-(dir.x),pos.z-(dir.z));
            temp[2]=getIUSHeight(pos.x+(dir.z),pos.z+(dir.x));
            temp[3]=getIUSHeight(pos.x-(dir.z),pos.z-(dir.x));            
      }
      
     



      void init(char *file){
                    texture_t data, datasmall;
                    log_txt("\n\n*******************************************\n\nInitialisiere Terrain\n");
                    FILE *tcfg;
                    tcfg=fopen(file,"rt");
                    
                    fscanf(tcfg,"LEVELFILE: %s\n",&data.name);
                    data=LoadTexture2(data.name);
                    fscanf(tcfg,"LEVELSMALL: %s\n",&datasmall.name);
                    datasmall=LoadTexture2(datasmall.name);
                    fscanf(tcfg,"MAPSCALE: %i\nHEIGHTSCALE: %i\nMAXDISTANCE: %i\n",&scale, &hscale, &MaxView);
                    
                    fscanf(tcfg,"DETAILTEX: %s\n",&detail.name);
                    detail=LoadTexture(detail.name);
                    fscanf(tcfg,"FLOORTEX: %s\n",&floor.name);
                    floor=LoadTexture(floor.name);
                    //fscanf(tcfg,"SKYTEX1: %s\nSKYTEX2: %s\nSKYTEX3: %s\nSKYTEX4: %s\nSKYTEX UP: %s\nSKYTEX DOWN: %s\n",sky.sky[0].name,sky.sky[1].name,sky.sky[2].name,sky.sky[3].name,sky.sky[4].name,sky.sky[5].name);
                    //fscanf(tcfg,"FOG? (0/1): %i\n",&fogging);
                    
                    //fscanf(tcfg,"WATER? (0/1): %i\n",&water);
                    //if(water==1) fscanf(tcfg,"WATERHEIGHT: %i\nWATERTEX: %s\n",&waterc.height,waterc.wtex.name);
                    fscanf(tcfg,"MODELNR: %i\n",&modelnr);
                    
                    
                    models=(CMD2Model *)malloc(modelnr*sizeof(CMD2Model));
                    
                    for(int x=0;x<modelnr;x++){
                            fscanf(tcfg,"MODELINFOFILE: %s\n",&models[x].name);
                            models[x].Load();
                    }
                                                
                    fclose(tcfg);

                    max=data.width;
                    maxsmall=datasmall.width;
                    dif=(int)max/maxsmall;
                    scalesmall=scale*dif;

                    log_txt("difference between big and small: %i\nScalesmall: %i",dif,scalesmall);
                    step=data.bpp/8;
                    log_txt("Speicher reservieren");
                    points=(short *)malloc(max*max*sizeof(short));
                    pointssmall=(short *)malloc(maxsmall*maxsmall*sizeof(short));

                    log_txt("Daten in Landschaftsspeicher schreiben, step=%i",step);

                    for(int x=0;x<max*max-1;x++){
                            points[x]=data.data[x*step]*hscale;

                    }
                    step=datasmall.bpp/8;
                    for(int x=0; x<maxsmall*maxsmall-1;x++){
                            pointssmall[x]=datasmall.data[x*step]*hscale;

                    }
                    

                    log_txt("Scale: %i\nHeightscale: %i",scale,hscale);


                    log_txt("Daten sortiert");

                    free(data.data);
                    free(datasmall.data);
                    
                    log_txt("Speicher freigegeben");
                    
                    glGenTextures(1,&shadows.texID);    
                    glBindTexture(GL_TEXTURE_2D,shadows.texID);
                
                      unsigned char* imgData=(unsigned char*)malloc(sizeof(unsigned char)*2048*2048*3);
                      glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,2048,2048,0,GL_RGB,GL_UNSIGNED_BYTE,imgData);
                      delete [] imgData;    
                
                
                      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
                      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
                      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
                      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

                    log_txt("Terrain initialisiert\n\n*********************************\n\n");



      }

      void renderTerrainDepth(){
           glColor3f(1.0,1.0,1.0);
           glDisable(GL_LIGHTING);
           
           
           glBegin(GL_QUADS);
           
           for(int x=0;x<max-2;x++){
                 
                   for(int z=0;z<max-2;z++){
                            
                            glColor3f(1.0,1.0,1.0);glVertex3f(x*scale,getHeight(x,z),-z*scale);

                            glColor3f(1.0,1.0,1.0);glVertex3f((x+1)*scale,getHeight(x+1,z),-z*scale);

                            glColor3f(1.0,1.0,1.0);glVertex3f((x+1)*scale,getHeight(x+1,z+1),-(z+1)*scale);

                            glColor3f(1.0,1.0,1.0);glVertex3f(x*scale,getHeight(x,z+1),-(z+1)*scale);
                          
                  }
                 

           }
           glEnd();
           
      }
      
      void renderTerrain(Vertex pos){
           //***************************************************
           //Drawing the Landscape
           //***************************************************
        if(fogging==1){
        glEnable(GL_FOG);
        float fc[4]={0.9,0.9,0.9,1.0};
	    glFogfv(GL_FOG_COLOR,fc);
	    glFogf(GL_FOG_START,0.25*MaxView);
	    glFogf(GL_FOG_END,4*MaxView);
        glFogf(GL_FOG_DENSITY,0.2);
	    glFogf(GL_FOG_MODE,GL_LINEAR);
        }

           glUnbindAll();
           glBindTex(detail,0);
           
           glDisable(GL_LIGHTING);

           glBegin(GL_QUADS);
           for(int x=0; x<maxsmall-2;x++){
                   
                   for(int z=0; z<maxsmall-2;z++){


                            if(fabs(pos.x-(x+1.2)*scalesmall)>MaxView || fabs(pos.z-(z+1.2)*scalesmall)>MaxView || fabs(pos.x-(x-1.2)*scalesmall)>MaxView || fabs(pos.z-(z-1.2)*scalesmall)>MaxView){
                            

                            
                            glTexCoordARB(float(x+1)/maxsmall,float(z+1)/maxsmall);

                            glVertex3f(x*scalesmall,getHeightSmall(x,z),-z*scalesmall);

                            
                            glTexCoordARB(float(x+2)/maxsmall,float(z+1)/maxsmall);

                            glVertex3f((x+1)*scalesmall,getHeightSmall(x+1,z),-z*scalesmall);

                           
                            glTexCoordARB(float(x+2)/maxsmall,float(z+2)/maxsmall);

                            glVertex3f((x+1)*scalesmall,getHeightSmall(x+1,z+1),-(z+1)*scalesmall);

                           
                            glTexCoordARB(float(x+1)/maxsmall,float(z+2)/maxsmall);

                            glVertex3f(x*scalesmall,getHeightSmall(x,z+1),-(z+1)*scalesmall);
                            }
                   }
                  
           }
           
           glEnd();
           
           short act=0;
            glUnbindAll();
           glBindTex(detail,0);
           
           glColor3f(1,1,1);
           glBegin(GL_QUADS);

           for(int x=0;x<max-2;x++){
                 
                   for(int z=0;z<max-2;z++){
                            if(fabs(pos.x-x*scale)<MaxView && fabs(pos.z-z*scale)<MaxView){
                            
                           
                            glTexCoordARB(float(x+1)/max,float(z+1)/max,0);
                            
                            glVertex3d(x*scale,getHeight(x,z),-z*scale);

                            
                            glTexCoordARB(float(x+2)/max,float(z+1)/max,0);
                            
                            glVertex3d((x+1)*scale,getHeight(x+1,z),-z*scale);

                           
                            glTexCoordARB(float(x+2)/max,float(z+2)/max,0);
                    
                            glVertex3d((x+1)*scale,getHeight(x+1,z+1),-(z+1)*scale);

                            
                            glTexCoordARB(float(x+1)/max,float(z+2)/max,0);
                            
                            glVertex3d(x*scale,getHeight(x,z+1),-(z+1)*scale);
                            
                            
                            }
                  }
                 

           }
           glEnd();
                       


           //******************************************************
           glUnbindAll();


           
        if(fogging==1) glDisable(GL_FOG);

      }




}; //Terrain

#endif



