/*
  Name: camera.h
  Author: Carsten Haubold (CarstenHaubold@web.de)
  Date: 14.02.05 20:43
  Description: Simple camera that can be moved through 3D-Worlds,
               it's for free use!
*/


#include <gl/gl.h>
#include <gl/glu.h>
#include "3dmath.h"

#ifndef _camera
#define _camera

class Camera{
public:
       Vertex pos; //actual Position
       float speed; //how fast we're moving in viewdir
       int max; //maximum movement in each direction
       float yrot,yaw; //rotation in y & old y-component of viewdir
       
       Camera(){
                pos.x=0;
                pos.y=0;
                pos.z=0;
                speed=0.0;
                max=200;
                yrot=90.0;
                yaw=0;      
                
       }


       void init(float x,float y,float z, float viewx, float viewy, float viewz, int mscale){
                  pos.x=x;
                  pos.y=y;
                  pos.z=z;
                  max=mscale;
       }
       
       
       void update(float fpsq=1){
                pos.x+=sin(DEG2RAD*yrot)*speed*fpsq;
                pos.y+=sin(DEG2RAD*yaw)*speed*fpsq;
                pos.z+=cos(DEG2RAD*yrot)*speed*fpsq;


                if(pos.x<1) pos.x=1;
                if(pos.z<1) pos.z=1;
                if(pos.y<1) pos.y=1;
                if(pos.x>max-1) pos.x=max-1;
                if(pos.z>max-1) pos.z=max-1;
               

       }
       
       void look_left(float amount=0.4){
            yrot-=amount;
       }
       
       void look_right(float amount=0.4){
            yrot+=amount;
       }
       
       void look_up(float amount=0.4){
            yaw+=amount;
       }
       
       void look_down(float amount=0.4){
            yaw-=amount;
       }
       
       void accelerate(float amount=0.4){
            speed+=amount;
       }
       
       void slowdown_reverse(float amount=0.4){
            speed-=amount;
       }
       
       void slowdown(float amount=0.4){
            speed-=amount;
            if(speed<0) speed=0;
       }
       
       void stop(){
            speed=0;
       }

       void positionCamera(){
            glRotatef(-yaw,1,0,0);
            glRotatef(yrot,0,1,0);
            glTranslatef(-pos.x,-pos.y,pos.z-5);
       }


private:

} ;

#endif






