// Includes
#ifndef _OPENGL_WIN
#define _OPENGL_WIN

#define WIN32_LEAN_AND_MEAN		// trim the excess fat from Windows
#define WIN32_EXTRA_LEAN

#include <windows.h>
#include <locale.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include "../framework/logging.h"
#include "../framework/texture.h"
#include "../framework/CTimer.h"
#include "../framework/3dmath.h"
#include "../framework/camera.h"
#include "../framework/md2.h"
#include "../framework/cbass.h"
#include "../framework/logo.h"
#include "../framework/menu.h"

#define MAXMODELS 3000


// Function Declarations

HDC        hDC=NULL;                        // Permanent Rendering Context
HWND        hWnd=NULL;                // Private GDI Device Context
HGLRC        hRC=NULL;                // Holds Our Window Handle

//bool        keys[256];                Key Array already defined in menu.h
bool        active=TRUE;                // Window Active Flag Set To TRUE By Default
bool        keys[256];
bool keytemp[256];
int menu_nr;
int menu_switch;
bool fullscreen=true;


FILE *config;

//From menu.h
//bool init_settings=true; //Do we still have to initialize settings?
//int resx,resy;//resolution X andY
//int b,f;//Bpp and Fullscreen
//bool        fullscreen=TRUE;        // Fullscreen Flag Set To Fullscreen Mode By Default
int mx,my;
LRESULT        CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);




void InitPositions();
int InitGL(GLvoid);
void getKeys();
int DrawGLScene(GLvoid);


//Gre festlegen
GLvoid ReSizeGLScene(GLsizei width, GLsizei height)        // Resize And Initialize The GL Window
{
        if (height==0)                                        // Prevent A Divide By Zero By
        {
                height=1;                                // Making Height Equal One
        }

        glViewport(0, 0, width, height);                // Reset The Current Viewport

        // Matrix festlegen
        glMatrixMode(GL_PROJECTION);                        // Select The Projection Matrix
        glLoadIdentity();                                // Reset The Projection Matrix

        // Calculate The Aspect Ratio Of The Window
        gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,9999999.0f);

        glMatrixMode(GL_MODELVIEW);                        // Select The Modelview Matrix
        glLoadIdentity();                                // Reset The Modelview Matrix
}

GLvoid KillGLWindow(GLvoid)                                        // Properly Kill The Window
{
        if (fullscreen)                                                // Are We In Fullscreen Mode?
                {

                        ChangeDisplaySettings(NULL,0);
                        ShowCursor(TRUE);                                // Show Mouse Pointer
                }
        if (hRC)                                                // Do We Have A Rendering Context?
                {
        if (!wglMakeCurrent(NULL,NULL))                        // Are We Able To Release The DC And RC Contexts?
                        {
                 log_msg("Release Of DC And RC Failed.");
                        }
        if (!wglDeleteContext(hRC))                        // Are We Able To Delete The RC?
                        {
                         log_msg("Release Rendering Context Failed.");
                        }
                        hRC=NULL;                                        // Set RC To NULL
                }
         if (hDC && !ReleaseDC(hWnd,hDC))                        // Are We Able To Release The DC
                {
                        log_msg("Release Device Context Failed.");
                        hDC=NULL;                                        // Set DC To NULL
                }
        if (hWnd && !DestroyWindow(hWnd))                        // Are We Able To Destroy The Window?
                {
                        log_msg("Could Not Release hWnd.");
                        hWnd=NULL;                                        // Set hWnd To NULL
                }

        killSound();
}//KillGLWindow


BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{

 GLuint                PixelFormat;                                // Holds The Results After Searching For A Match
 HINSTANCE        hInstance;                                // Holds The Instance Of The Application
 WNDCLASS        wc;                                        // Windows Class Structure
 DWORD                dwExStyle;                                // Window Extended Style
 DWORD                dwStyle;                                // Window Style
 fullscreen=fullscreenflag;                                // Set The Global Fullscreen Flag


hInstance                = GetModuleHandle(NULL);                // Grab An Instance For Our Window
        wc.style                = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;        // Redraw On Move, And Own DC For Window
        wc.lpfnWndProc                = (WNDPROC) WndProc;                        // WndProc Handles Messages
        wc.cbClsExtra                = 0;                                        // No Extra Window Data
        wc.cbWndExtra                = 0;                                        // No Extra Window Data
        wc.hInstance                = hInstance;                                // Set The Instance
        wc.hIcon                = LoadIcon(NULL, IDI_WINLOGO);                // Load The Default Icon
        wc.hCursor                = LoadCursor(NULL, IDC_ARROW);                // Load The Arrow Pointer
        wc.hbrBackground        = NULL;                                        // No Background Required For GL
        wc.lpszMenuName                = NULL;                                        // We Don't Want A Menu
        wc.lpszClassName        = "OpenGL";                                // Set The Class Name


if (!RegisterClass(&wc))                                        // Attempt To Register The Window Class
        {
                log_msg("Failed To Register The Window Class.");
                return FALSE;                                                // Exit And Return FALSE
        }

 
  if (fullscreen)                                                        // Attempt Fullscreen Mode?
        {
    DEVMODE dmScreenSettings;                                // Device Mode
                memset(&dmScreenSettings,0,sizeof(dmScreenSettings));        // Makes Sure Memory's Cleared
                dmScreenSettings.dmSize=sizeof(dmScreenSettings);        // Size Of The Devmode Structure
                dmScreenSettings.dmPelsWidth        = width;                // Selected Screen Width
                dmScreenSettings.dmPelsHeight        = height;                // Selected Screen Height
                dmScreenSettings.dmBitsPerPel        = bits;                        // Selected Bits Per Pixel
                dmScreenSettings.dmDisplayFrequency = 75; //75 Hz
                dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
 // Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
                if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)

                {
// If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
                        if (log_yn("The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?"))
                        {
                                        fullscreen=FALSE;                        // Select Windowed Mode (Fullscreen=FALSE)
                        }
                        else
                        {
        // Pop Up A Message Box Letting User Know The Program Is Closing.
                                log_msg("Program Will Now Close.");
                                return FALSE;                                // Exit And Return FALSE
                        }
                }
        }      
#ifndef CW_USEDEFAULT
#define CW_USEDEFAULT 200
#endif

int pos_x=0;
int pos_y=0;
 if (fullscreen)                                                        // Are We Still In Fullscreen Mode?
        {
        dwExStyle=WS_EX_APPWINDOW;                                // Window Extended Style
                dwStyle=WS_POPUP;                                        // Windows Style
                ShowCursor(FALSE);                                        // Hide Mouse Pointer
        }
        else
        {
                dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;                // Window Extended Style
                dwStyle=WS_OVERLAPPEDWINDOW;                                // Windows Style
                pos_x=CW_USEDEFAULT;
                pos_y=CW_USEDEFAULT;

        }

if (!(hWnd=CreateWindowEx(        dwExStyle,                        // Extended Style For The Window
                                        "OpenGL",                        // Class Name
                                        title,                                // Window Title
                                        WS_CLIPSIBLINGS |                // Required Window Style
                                        WS_CLIPCHILDREN |                // Required Window Style
                                        dwStyle,                        // Selected Window Style
                                        pos_x, pos_y,                                // Window Position
                                        width, height,                        // Selected Width And Height
                                        NULL,                                // No Parent Window
                                        NULL,                                // No Menu
                                        hInstance,                        // Instance
                                        NULL)))                                // Don't Pass Anything To WM_CREATE

                                        {
                KillGLWindow();                                                // Reset The Display
                log_msg("Window Creation Error.");
                return FALSE;                                                // Return FALSE
        }
static        PIXELFORMATDESCRIPTOR pfd=                                // pfd Tells Windows How We Want Things To Be
        {
                sizeof(PIXELFORMATDESCRIPTOR),                                // Size Of This Pixel Format Descriptor
                1,                                                        // Version Number
                PFD_DRAW_TO_WINDOW |                                        // Format Must Support Window
                PFD_SUPPORT_OPENGL |                                        // Format Must Support OpenGL
                PFD_DOUBLEBUFFER,                                        // Must Support Double Buffering
                PFD_TYPE_RGBA,                                                // Request An RGBA Format
                bits,                                                        // Select Our Color Depth
                0, 0, 0, 0, 0, 0,                                        // Color Bits Ignored
                0,                                                        // No Alpha Buffer
                0,                                                        // Shift Bit Ignored
                0,                                                        // No Accumulation Buffer
                0, 0, 0, 0,                                                // Accumulation Bits Ignored
                24,                                                        // 16Bit Z-Buffer (Depth Buffer)
                16,                                                        // No Stencil Buffer
                0,                                                        // No Auxiliary Buffer
                PFD_MAIN_PLANE,                                                // Main Drawing Layer
                0,                                                        // Reserved
                0, 0, 0                                                        // Layer Masks Ignored
        };


        if (!(hDC=GetDC(hWnd)))                                                // Did We Get A Device Context?
        {
                KillGLWindow();                                                // Reset The Display
                log_msg("Can't Create A GL Device Context.");
                return FALSE;                                                // Return FALSE
        }

         if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))                        // Did Windows Find A Matching Pixel Format?
        {
                KillGLWindow();                                                // Reset The Display
                log_msg("Can't Find A Suitable PixelFormat.");
                return FALSE;                                                // Return FALSE
        }

        if(!SetPixelFormat(hDC,PixelFormat,&pfd))                        // Are We Able To Set The Pixel Format?
        {
                KillGLWindow();                                                // Reset The Display
                log_msg("Can't Set The PixelFormat.");
                return FALSE;                                                // Return FALSE
        }

        if (!(hRC=wglCreateContext(hDC)))                                // Are We Able To Get A Rendering Context?
        {
                KillGLWindow();                                                // Reset The Display
                log_msg("Can't Create A GL Rendering Context.");
                return FALSE;                                                // Return FALSE
        }

        if(!wglMakeCurrent(hDC,hRC))                                        // Try To Activate The Rendering Context
        {
                KillGLWindow();                                                // Reset The Display
                log_msg("Can't Activate The GL Rendering Context.");
                return FALSE;                                                // Return FALSE
        }

        ShowWindow(hWnd,SW_SHOW);                                        // Show The Window
        SetForegroundWindow(hWnd);                                        // Slightly Higher Priority
        SetFocus(hWnd);                                                        // Sets Keyboard Focus To The Window
        ReSizeGLScene(width, height);                                        // Set Up Our Perspective GL Screen
        
        log_txt("InitGL:");
        if(!InitGL())                                                        // Initialize Our Newly Created GL Window
        {
                    KillGLWindow();                                                // Reset The Display
                    log_msg("Initialization Failed.");
                    return FALSE;                                                // Return FALSE
        }
        

                return TRUE;                                                        // Success
}

LRESULT CALLBACK WndProc(        HWND        hWnd,                        // Handle For This Window
                                UINT        uMsg,                        // Message For This Window
                                WPARAM        wParam,                        // Additional Message Information
                                LPARAM        lParam)                        // Additional Message Information
{

switch (uMsg)                                                // Check For Windows Messages
        {
case WM_ACTIVATE:                                // Watch For Window Activate Message
                {
                        if (!HIWORD(wParam))                        // Check Minimization State
                        {
                                active=TRUE;                        // Program Is Active
                        }
                        else
                        {
                                active=FALSE;                        // Program Is No Longer Active
                        }

                        return 0;                                // Return To The Message Loop
                }
case WM_SYSCOMMAND:                                // Intercept System Commands
                {
                        switch (wParam)                                // Check System Calls
                        {
                                case SC_SCREENSAVE:                // Screensaver Trying To Start?
                                case SC_MONITORPOWER:                // Monitor Trying To Enter Powersave?
                                return 0;                        // Prevent From Happening
                        }
                        break;                                        // Exit
                }
case WM_CLOSE:                                        // Did We Receive A Close Message?
                {
                        PostQuitMessage(0);                        // Send A Quit Message
                        return 0;                                // Jump Back
                }
case WM_KEYDOWN:                                // Is A Key Being Held Down?
                {
                        keys[wParam] = TRUE;                        // If So, Mark It As TRUE
                        return 0;                                // Jump Back
                }
case WM_KEYUP:                                        // Has A Key Been Released?
                {
                        keys[wParam] = FALSE;                        // If So, Mark It As FALSE
                        return 0;                                // Jump Back
                }

case WM_MOUSEMOVE:
    {

      my = LOWORD(lParam);
      mx = HIWORD(lParam);

      return 0;
    }
 case WM_SIZE:                                        // Resize The OpenGL Window
                {
                        ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));  // LoWord=Width, HiWord=Height
                        
                        return 0;                                // Jump Back
                }
        }
   // Pass All Unhandled Messages To DefWindowProc
        return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

//Main routine
int WINAPI WinMain(        HINSTANCE        hInstance,                // Instance
                        HINSTANCE        hPrevInstance,                // Previous Instance
                        LPSTR                lpCmdLine,                // Command Line Parameters
                        int                nCmdShow)                // Window Show State
{

        MSG        msg;                                                // Windows Message Structure
        BOOL        done=FALSE;    
        log_init("3D-Caste");                                  
        // Fenster erstellen!!!!!!!!!!!
        if (!menu.CreateGLWindow("3D-Caste",600,400,16,0))
        {
                return 0;                                                // Quit If Window Was Not Created
        }
        if(init_quit) return 0;
        ShowCursor(false);
        if (!CreateGLWindow("3D-Caste",resx,resy,b,f))
        {
                return 0;                                                // Quit If Window Was Not Created
        }
        //   HAUPTSCHLEIFE!!
        done=false;
        while(!done)                                                        // Loop That Runs Until done=TRUE
        {
        if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))                // Is There A Message Waiting?
                {
                if (msg.message==WM_QUIT)                        // Have We Received A Quit Message?
                        {
                                done=TRUE;                                // If So done=TRUE
                        }
                        else                                                // If Not, Deal With Window Messages
                        {
                                TranslateMessage(&msg);                        // Translate The Message
                                DispatchMessage(&msg);                        // Dispatch The Message
                        }
                }
                else                                                        // If There Are No Messages
                {

                // Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
                        if (active && keys[VK_ESCAPE])        // Updating View Only If Active
                        {
                              done=TRUE;                                        // ESC or DrawGLScene Signalled A Quit
                        }
                        else                                                        // Not Time To Quit, Update Screen
                        {
                                DrawGLScene();
                                //DrawGLScene();
                                SwapBuffers(hDC);                                // Swap Buffers (Double Buffering)
                        }

	       }
}
}

#endif


